#include "deltaplugin.h"
#include "deltaapi.h"
#include "object.h"
#include <new>

void DloClearPointers();
void __cdecl DloCall(void* input, void* output);

static void __BUILDTIME_TYPE_CHECKS__()
{
    // IF A LINE IN THIS SECTION FAILS, THAT TYPE HAS AN INCORRECT TYPE SIZE!!
    BUILD_BUG_ON((((signed int)-1)>>1) != ((signed int)-1)); // Must have Arithmetic Signed Right Shifts.
    BUILD_BUG_ON(sizeof(void*) != 4); // 32-bit pointers only
    BUILD_BUG_ON(sizeof(NUM) != 8);
    BUILD_BUG_ON(sizeof(FLO) != 8);
}
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    return TRUE;
}

static CustomObject*	selection_ = NULL;
static void __cdecl (*ApiCall_)(void* input, void* output) = NULL;

ByteCode ApiCall(ByteCode input)
{
    ByteCode result;
    ApiCall_(&input, &result);
    return result;
}


void DLL_EXPORT DeltaDloInit(
    void __cdecl (*setup)(
        void* lucidstring, 
        void* objectname,
        void* private_data_size, //encoded integer
        void __cdecl (*dlo_call)(void* input, void* output)
    ), 
    void __cdecl (*api_call)(void* input, void* output)
){
    __BUILDTIME_TYPE_CHECKS__();
    DloClearPointers();
    setup((void*)CustomObject::OBJ_LUCID, (void*)CustomObject::OBJ_NAME, (void*)sizeof(CustomObject), &DloCall);
    ApiCall_ = api_call;
}
void DLL_EXPORT DeltaDloRepo()
{
    
}
void dlo_setup(NUM object_type_id){
    CustomObject::SetupRoutine(object_type_id);
}
void dlo_cleanup(){
    CustomObject::CleanupRoutine();
}
void dlo_select(void* data){
    selection_ = (CustomObject*)data;
}
void dlo_init(){
    if(selection_){
        new (selection_) CustomObject(); //required in case compiler stores extra non-member data
        selection_->Create();
    }
}
void dlo_copy(void* data){
    //here DELTA provides the memory buffer by first calling dlo_init()
    //and then calls this routine to copy the member data over.
    if(selection_){
        *((CustomObject*)selection_) = *((CustomObject*)data);
        selection_->CopyFrom(*((CustomObject*)data));
    }
}
void dlo_repo(){
    if(selection_){
        selection_->Delete();
        selection_->~CustomObject(); //paired with the placement new call (above)
    }
}
void dlo_proc(){
    if(selection_){
        selection_->Proc();
    }
}
void dlo_resp(){
    if(selection_){
        selection_->Resp();
    }
}
void dlo_draw(FLO delta){
    if(selection_){
        selection_->Draw(delta);
    }
}
void dlo_coll(OBJID other_object, POS coll_x, POS coll_y, NUM boxid, NUM flags){
    if(selection_){
        selection_->CollidesWith(other_object, coll_x, coll_y, boxid, flags);
    }
}
void dlo_proc2(){
    if(selection_){
        selection_->ProcInEditor();
    }
}
void dlo_draw2(FLO delta){
    if(selection_){
        selection_->DrawInEditor(delta);
    }
}
void dlo_rend(){
    if(selection_){
        selection_->Rend();
    }
}
void dlo_rend2(){
    if(selection_){
        selection_->RendInEditor();
    }
}
//##############################################################################
//##############################################################################

#define Num(x) (b->i(x))
#define Flo(x) (b->f(x))
#define Void(x) x

typedef void (*_api_funcptr)(ByteCode* args, ByteCode* ret);
void api_dlo_empty(ByteCode* b, ByteCode* r){}

void api_dlo_setup(ByteCode* b, ByteCode* r){
    Void(dlo_setup(Num(0)));
}
void api_dlo_cleanup(ByteCode* b, ByteCode* r){
    Void(dlo_cleanup());
}
void api_dlo_select(ByteCode* b, ByteCode* r){
    Void(dlo_select((void*)(U32)(Num(0))));
}
void api_dlo_init(ByteCode* b, ByteCode* r){
    Void(dlo_init());
}
void api_dlo_copy(ByteCode* b, ByteCode* r){
    Void(dlo_copy((void*)(U32)(Num(0))));
}
void api_dlo_repo(ByteCode* b, ByteCode* r){
    Void(dlo_repo());
}
void api_dlo_proc(ByteCode* b, ByteCode* r){
    Void(dlo_proc());
}
void api_dlo_resp(ByteCode* b, ByteCode* r){
    Void(dlo_resp());
}
void api_dlo_draw(ByteCode* b, ByteCode* r){
    Void(dlo_draw(Flo(0)));
}
void api_dlo_coll(ByteCode* b, ByteCode* r){
    Void(dlo_coll(OBJID(Num(0)),Flo(1),Flo(2),Num(3),Num(4)));
}
void api_dlo_proc2(ByteCode* b, ByteCode* r){
    Void(dlo_proc2());
}
void api_dlo_draw2(ByteCode* b, ByteCode* r){
    Void(dlo_draw2(Flo(0)));
}
void api_dlo_rend(ByteCode* b, ByteCode* r){
    Void(dlo_rend());
}
void api_dlo_rend2(ByteCode* b, ByteCode* r){
    Void(dlo_rend2());
}
//##############################################################################
//##############################################################################

_api_funcptr dlo_api_table[2][256] = {

//##############################################################################
//##############################################################################
// 00 - NULL (Reserved for the scripting engine!)
{((_api_funcptr)(1))},

//##############################################################################
//##############################################################################
// 01 - Delta Level Object (Management Functions)
{&api_dlo_empty,
&api_dlo_setup,
&api_dlo_cleanup,
&api_dlo_select,
&api_dlo_init,
&api_dlo_copy,
&api_dlo_repo,
&api_dlo_proc,
&api_dlo_resp,
&api_dlo_draw,
&api_dlo_coll,
&api_dlo_proc2,
&api_dlo_draw2,
&api_dlo_rend,
&api_dlo_rend2,
((_api_funcptr)(1))}};

void __cdecl DloCall(void* input, void* output)
{
    dlo_api_table[((ByteCode*)input)->GetA() & 0x01]
                 [((ByteCode*)input)->GetB() & 0xFF]
                 ((ByteCode*)input, (ByteCode*)output);
}
void DloClearPointers()
{
    for(INT i = 0; i < 2; i++){
        BIT is_filled = true;
        for(INT j = 0; j < 256; j++){
            if(dlo_api_table[i][j] == ((_api_funcptr)(1))) is_filled = false;
            if(!is_filled) dlo_api_table[i][j] = &api_dlo_empty;
        }
    }
}

