#include "object.h"
#include "deltaapi.h"
#include "math.h"
/*
 *  Simion32 says:
 *    This is the main code file. Most of your custom object code goes here.
 */

const char* CustomObject::OBJ_NAME = "PouncePanel"; // whatever you want, shows up in the level editor
const char* CustomObject::OBJ_LUCID = "7vmPGxbuHbCrqwyKfDxrFN3i8PLgzdBR"; //identifies your object type
DGEID       CustomObject::OBJ_DGEID = 0xFFFFFFFF; //you can't get this number until the SetupRoutine() runs

void CustomObject::SetupRoutine(DGEID your_object_type_id)
{	// Called by DELTA just after your object type is loaded.
    // You should load in and save any required object type IDs here (based on the object LUCIDs).
    // Be careful -- you can only save/use static variables here.
	OBJ_DGEID = your_object_type_id;
    
}
void CustomObject::CleanupRoutine()
{	// Called by DELTA when the object type is no longer needed in-game.
    // If you allocated any memory manually in the SetupRoutine(), 
    // you will need to delete/free that memory here.
	// Be careful -- you can only use static variables here.
    
}
void CustomObject::Create()
{	// Called by DELTA to create an instance of your object.
	sfx_id_ = resources_get_engine_id("7vgg-zIlHbFUa46p6wsZ5PA4fyNB7fIF",resources_get_filetype("dss"));
	
	OBJID o = object_get_id();
	object_set_int(o, 'f', (object_get_int(o,'f') & ~(U64)OBJECT_FLAG_USE_PHYSICS));
	if(object_get_flo(o, 0x01) == 0.0) object_set_flo(o, 0x01, 8.0);
	
	within_ = 0xFFFFFFFF;
	previous_ = 0xFFFFFFFF;
}
void CustomObject::CopyFrom(CustomObject& other)
{	// Called by DELTA to copy an instance of your object.
	// Putting copy code here is ONLY needed when you are
	// allocating your own memory (new/delete, malloc/free).
	
}
void CustomObject::Delete()
{	// Called by DELTA to destroy an instance of your object.
	
}
void CustomObject::Proc()
{	// This is where you do game logic.
	// Proc() is ALWAYS called exactly 60 times per second.
	
	OBJID o = object_get_id();
    object_set_animation_speed(o, 0.0);
    ///object_set_frame_number(o,object_get_flo(o,0x00)*4.0);
	
	FLO xv = 0.0;
    FLO yv = 0.0;
    FLO theta = 0.0;
    if((o > 0) && (object_get_int(o-1, 'f') & OBJECT_FLAG_IS_PAYLOAD)){
        FLO dx = (object_get_flo(o-1, 'x') - object_get_flo(o, 'x'));
        FLO dy = (object_get_flo(o-1, 'y') - object_get_flo(o, 'y'));
        theta = atan2(dy,dx);
        xv = cos(theta) * object_get_flo(o, 0x01);
        yv = sin(theta) * object_get_flo(o, 0x01);
    }
    
    object_set_frame_number(o,(((theta / M_PI) * 8.0) - 0.5) * 4.0);
    
    
	if(within_ != 0xFFFFFFFF){
        object_set_position(within_, object_get_flo(o,'x'), object_get_flo(o,'y'));
        object_set_animation(within_, "Pounce");
        object_set_animation_speed(within_, 2.0);
        object_set_flags(within_, object_get_flags(within_) | 1);
        object_set_int(within_, 'f', (object_get_int(within_,'f') & ~(U64)OBJECT_FLAG_USE_PHYSICS));
        
        if(input_qbtn(xPAD_B,1) == 1){
            audio_sfx_play(sfx_id_, (1), 0, 1.0, 0.0, -1);
            object_set_flo(within_, 'U', xv);
            object_set_flo(within_, 'V', yv);
            previous_ = within_;
            within_ = 0xFFFFFFFF;
            
        }
        
	}
	if(previous_ != 0xFFFFFFFF){
        if((fabs(object_get_flo(o,'x') - object_get_flo(previous_,'x')) >= 64.0)
        || (fabs(object_get_flo(o,'y') - object_get_flo(previous_,'y')) >= 64.0)){
            previous_ = 0xFFFFFFFF;
        }
	}
}	
void CustomObject::Resp()
{	// This is where you respond to any game events that were triggered.
	// Resp() is ALWAYS called exactly 60 times per second, comes after Proc().
	
	
}
void CustomObject::Rend()
{	// This is where you tell DELTA to render graphics. 
	// Rend() is ALWAYS called exactly 60 times per second, comes after Resp().
	// Please DO NOT do any game logic here, it's bad design!
	// This is where history saving is done for proper Draw() usage.
	// This is where you animate the sprite.
	
}
void CustomObject::Draw(FLO delta)
{	// This function is for advanced bitmap drawing, before the rendering takes place.
    // You can use this to program custom buffer effects or even use your own bitmap renderer.
	// Draw() is called before every screen update, using the delta value to blend frame data.
	// The delta value is 0 to 1 with 0 meaning the previous logic frame and 1 meaning the current.
	//! To show a bitmap, you need to call surface_render().
	//! All DELTA bitmaps are 0xAARRGGBB formatted.
	
	
	
}
void CustomObject::CollidesWith(OBJID other_object, POS coll_x, POS coll_y, NUM boxid, NUM flags)
{	// This is where you make your object react to collision with other objects!
	// Will ONLY be called when there is an object-vs-object collision.
	//! If collision flags are both nonzero, the flags must 
	//! overlap (AND is nonzero) for a collision to happen.
	//! Flag based collision boxes cannot collide with non-flag ones.
	
	if(other_object != previous_){
        within_ = other_object;
	}
	
	//object_delete(other_object);
	
	
	
}
void CustomObject::ProcInEditor()
{   // This function is like Proc(), but is called by the editor.
	
	OBJID o = object_get_id();
    object_set_animation_speed(o, 0.0);
    ///object_set_frame_number(o,object_get_flo(o,0x00)*4.0);
    
	FLO theta = 0.0;
    if((o > 0) && (object_get_int(o-1, 'f') & OBJECT_FLAG_IS_PAYLOAD)){
        FLO dx = (object_get_flo(o-1, 'x') - object_get_flo(o, 'x'));
        FLO dy = (object_get_flo(o-1, 'y') - object_get_flo(o, 'y'));
        theta = atan2(dy,dx);
    }
    
    object_set_frame_number(o,(((theta / M_PI) * 8.0) - 0.5) * 4.0);
    
    
}
void CustomObject::DrawInEditor(FLO delta)
{	// This function is like Draw(), but is called by the editor.
	
}
void CustomObject::RendInEditor()
{   // This function is like Rend(), but is called by the editor.
	
}
