#include "object.h"
#include "deltaapi.h"
#include "math.h"
/*
 *  Simion32 says:
 *    This is the main code file. Most of your custom object code goes here.
 */

const char* CustomObject::OBJ_NAME = "Stix"; // whatever you want, shows up in the level editor
const char* CustomObject::OBJ_LUCID = "7vgUm7WoHbEYe51ylSWC3V8MUzJmv-CS"; //identifies your object type
DGEID       CustomObject::OBJ_DGEID = 0xFFFFFFFF; //you can't get this number until the SetupRoutine() runs

void CustomObject::SetupRoutine(DGEID your_object_type_id)
{	// Called by DELTA just after your object type is loaded.
    // You should load in and save any required object type IDs here (based on the object LUCIDs).
    // Be careful -- you can only save/use static variables here.
	OBJ_DGEID = your_object_type_id;
    
}
void CustomObject::CleanupRoutine()
{	// Called by DELTA when the object type is no longer needed in-game.
    // If you allocated any memory manually in the SetupRoutine(), 
    // you will need to delete/free that memory here.
	// Be careful -- you can only use static variables here.
    
}
void CustomObject::Create()
{	// Called by DELTA to create an instance of your object.
	OBJID o = object_get_id();
    object_set_flo(o, 'V', 3.0);
    object_set_flo(o, 'Y', 3.0);
	sfx_id_ = resources_get_engine_id("7vgg-zIlHbFUa46p6wsZ5PA4fyNB7fIF",resources_get_filetype("dss"));
	font_id_ = resources_load_engine_id("7v3ZQ440N64zKwe63yNPl5ZOuR48Dchm",resources_get_filetype("dft"));
	
	message_active_ = false;
	
	camera_set_smoothing_level(16.0);
}
void CustomObject::CopyFrom(CustomObject& other)
{	// Called by DELTA to copy an instance of your object.
	// Putting copy code here is ONLY needed when you are
	// allocating your own memory (new/delete, malloc/free).
	
}
void CustomObject::Delete()
{	// Called by DELTA to destroy an instance of your object.
	
}
void CustomObject::Proc()
{	// This is where you do game logic.
	// Proc() is ALWAYS called exactly 60 times per second.
	OBJID o = object_get_id();
	camera_center_on_object(o, 0.0, 0.0);
	if(~object_get_flags(o) & 1){
        object_set_flo(o, 'X', 0.0);
        object_set_animation(o, "Stand");
        if(!input_btn(xPAD_Y,1)){
            if(input_btn(xPAD_RIGHT,1)){
                object_set_flo(o, 'X', 2.0);
                object_set_animation(o, "Walk");
                object_set_int(o, 'f', object_get_int(o, 'f') & ~OBJECT_FLAG_FLIP_X);
            }
            if(input_btn(xPAD_LEFT,1)){
                object_set_flo(o, 'X', -2.0);
                object_set_animation(o, "Walk");
                object_set_int(o, 'f', object_get_int(o, 'f') | OBJECT_FLAG_FLIP_X);
            }
        }else{
            if(input_btn(xPAD_RIGHT,1)){
                object_set_flo(o, 'X', 3.0);
                object_set_animation(o, "Run");
                object_set_int(o, 'f', object_get_int(o, 'f') & ~OBJECT_FLAG_FLIP_X);
            }
            if(input_btn(xPAD_LEFT,1)){
                object_set_flo(o, 'X', -3.0);
                object_set_animation(o, "Run");
                object_set_int(o, 'f', object_get_int(o, 'f') | OBJECT_FLAG_FLIP_X);
            }
        }
        if(object_get_int(o, 'f') & OBJECT_FLAG_FLIP_X){
            camera_center_on_object(o, -80.0, 0.0);
        }else{
            camera_center_on_object(o, 80.0, 0.0);
        }
        if(!jump_status_ && (input_qbtn(xPAD_B, 1) == 1)){
            object_set_flo(o, 'V', -7.0);
            object_set_flo(o, 'Y', -7.0);
            audio_sfx_play(sfx_id_, 0, 0, 1.0, 0.0, -1);
            jump_status_ = 1;
        }
        if(jump_status_ == 1){
            object_set_animation(o, "Jump");
        }
        if(!input_btn(xPAD_B, 1)){
            FLO val = object_get_flo(o, 'V')+0.4375;
            object_set_flo(o, 'V', val);
            object_set_flo(o, 'Y', val);
        }else{
            FLO val = object_get_flo(o, 'V')+0.28125;
            object_set_flo(o, 'V', val);
            object_set_flo(o, 'Y', val);
        }
	}else{
        //stix is interacting with a pounce panel!!
        if(object_get_int(o, 'f') & OBJECT_FLAG_FLIP_X){
            camera_center_on_object(o, -80.0, 0.0);
        }else{
            camera_center_on_object(o, 80.0, 0.0);
        }
        Vx<FLO> r = physics_move_to_point(
            object_get_flo(o, 'x')+object_get_flo(o, 'U'), 
            object_get_flo(o, 'y')+object_get_flo(o, 'V'));
        object_set_flo(o, 'x', r[2]);
        object_set_flo(o, 'y', r[3]);
	}
	
}	
void CustomObject::Resp()
{	// This is where you respond to any game events that were triggered.
	// Resp() is ALWAYS called exactly 60 times per second, comes after Proc().
	OBJID o = object_get_id();
	if(object_respond_to_event(o,0)){
        object_set_flags(o, object_get_flags(o) & ~1LL);
        object_set_int(o, 'f', (object_get_int(o,'f') | (U64)OBJECT_FLAG_USE_PHYSICS));
        object_set_flo(o, 'Y', 3.0);
        object_set_flo(o, 'V', 3.0);
        object_set_animation(o, "Stand");
        if(!input_btn(xPAD_Y,1)){
            if(input_btn(xPAD_RIGHT,1)){
                object_set_flo(o, 'X', 2.0);
                object_set_animation(o, "Walk");
            }
            if(input_btn(xPAD_LEFT,1)){
                object_set_flo(o, 'X', -2.0);
                object_set_animation(o, "Walk");
            }
        }else{
            if(input_btn(xPAD_RIGHT,1)){
                object_set_flo(o, 'X', 3.0);
                object_set_animation(o, "Run");
            }
            if(input_btn(xPAD_LEFT,1)){
                object_set_flo(o, 'X', -3.0);
                object_set_animation(o, "Run");
            }
        }
        jump_status_ = 0;
	}
	if(object_respond_to_event(o,1)){
        object_set_flo(o, 'Y', 3.0);
        object_set_flo(o, 'V', 3.0);
        jump_status_ = 0;
	}
	if(~object_get_flags(o) & 1){
        object_set_animation_speed(o, object_get_flo(o,'U')*0.5);
	}
	
	
}
void CustomObject::Rend()
{	// This is where you tell DELTA to render graphics. 
	// Rend() is ALWAYS called exactly 60 times per second, comes after Resp().
	// Please DO NOT do any game logic here, it's bad design!
	// This is where history saving is done for proper Draw() usage.
	// This is where you animate the sprite.
	
}
void CustomObject::Draw(FLO delta)
{	// This function is for advanced bitmap drawing, before the rendering takes place.
    // You can use this to program custom buffer effects or even use your own bitmap renderer.
	// Draw() is called before every screen update, using the delta value to blend frame data.
	// The delta value is 0 to 1 with 0 meaning the previous logic frame and 1 meaning the current.
	//! To show a bitmap, you need to call surface_render().
	//! All DELTA bitmaps are 0xAARRGGBB formatted.
	
	
	
}
void CustomObject::CollidesWith(OBJID other_object, POS coll_x, POS coll_y, NUM boxid, NUM flags)
{	// This is where you make your object react to collision with other objects!
	// Will ONLY be called when there is an object-vs-object collision.
	//! If collision flags are both nonzero, the flags must 
	//! overlap (AND is nonzero) for a collision to happen.
	//! Flag based collision boxes cannot collide with non-flag ones.
	
}
void CustomObject::ProcInEditor()
{   // This function is like Proc(), but is called by the editor.
	
}
void CustomObject::DrawInEditor(FLO delta)
{	// This function is like Draw(), but is called by the editor.
	
}
void CustomObject::RendInEditor()
{   // This function is like Rend(), but is called by the editor.
	
}
